/****************************************************************************
    DAC.c
    -----
    bungsprojekt zum Ausgeben einer analogen Spannung ueber ein
    Widerstandsnetzwerk an Port B.
    Ueber den UART werden Zeichen empfangen.Handelt es sich um eine
    hexadezimale Ziffer ('0' bis 'A'), dann wird der entsprechende
    Zahlenwert (0 bis 15) auf Port B ausgegeben.

    Autor:    Christian Schifferle
    Datum:    November 2002
    Software: AVR-GCC
    needed
****************************************************************************/

#include <io.h>	// Wir brauchen Zugriff auf die I/O's des Controllers

#define F_CPU            8000000      /* 8Mhz */
#define UART_BAUD_RATE      9600      /* 9600 baud */

// Berechnung des Inhalts des UBBR-Registers
#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)

// Datentypdefinitionen
typedef unsigned char	BYTE;
typedef unsigned short	WORD;



int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{
    register BYTE c;    // Empfangenes Zeichen
    register BYTE x;	// Auszugebender Wert

    /* UART Control Register laden. Wir verwenden (immer noch)
       keine Interrupts. Hier wollen wir nur empfangen, also wird
       nur das Receiver Enable Flag gesetzt. */
    outp ((1 << RXEN), UCR);

    /* Baudrate einstellen */
    outp ((BYTE) UART_BAUD_SELECT, UBRR);          

    // Untere 4 Bits von Port B als Ausgang konfigurieren
    outp (0x0f, DDRB);

    for (;;) {
        // Prfen, ob ein Zeichen empfangen wurde.

        if (inp (USR) & (1 << RXC)) {     // Bit RXC im USR gesetzt ?
            // Ausgabewert initialisieren auf -1, weil BYTE mit 0xff
            x = 0xff;

            // Zeichen einlesen
            c = inp (UDR);

            // und pruefen, ob Hexziffer
            if (c >= '0' && c <= '9') {
                x = c - '0';
            }
            else if (c >= 'A' && c <= 'F') {
                x = c - 'A' + 10;
            }
            else if (c >= 'a' && c <= 'f') {
                x = c - 'a' + 10;
            }

            // Wenn eine gueltige Hexziffer empfangen wurde
            if (x != 0xff) {
               // Wert ausgeben
               outp (x, PORTB);
            }

        }
    }

} /* end of main */
