#include <io.h>

typedef unsigned short WORD;
typedef unsigned char  BYTE;


WORD Poti (uint8_t bDDR, uint8_t bPORT, uint8_t bPIN, uint8_t pPin)
/**********************************************************************
    Liest die Stellung eines vernderlichen Widerstandes und gibt einen
    entsprechenden Zahlenwert zurck.
    Zur Messung wird ein Kondensator zwischen Vcc und den Pin
    geschaltet. Zwischen den Pin und Masse wird der zu messende
    Widerstand geschaltet.

    in   : bDDR   Datenrichtungsregister (DDRx) des Ports.
    in   : bPORT  Portregister (PORTx) des Ports.
    in   : bPIN   Dateneingangsadresse (PINx) des Ports.
    in   : pPin   PinNummer, an welchem das Poti angeschlossen ist.

    return: Zhlerwert
            65535, wenn der zu messende Widerstand zu gross ist.
            0, wenn der zu messende Widerstand zu klein ist.
*/
{
    volatile WORD   x;
    register WORD   y;
    register BYTE   msk;    // Hilfsvariable zur Erstellung einer Bitmaske

    msk = 1 << pPin;         // Bitmaske definieren

    // Portpin auf Ausgang setzen. Leider knnen mit sbi und cbi nur
    // konstante Argumente angegeben werden, deswegen memory mapped io..
    __mmio (bDDR) = __mmio (bDDR) | msk;

    // Portpin auf HIGH setzen
    __mmio (bPORT) = __mmio (bPORT) | msk;

    /* Kondensator entladen, indem der Ausgang auf HIGH (Vcc) gesetzt wird.
       Die Spannungsdifferenz an den Kondensatorplatten wird so 0 V und
       der Kondensator entladen.
       Die Werte fr x wurden empirisch
       ermittelt bei einer CPU-Taktfrequenz von 8 MHz und einem
       Kondensatorwert von 0.1 uF */
    for (x=0; x<50;) x++ /* Zaehlen */;

    y = 0;

    /* Vorsichtshalber mal den internen Pull-Up ausschalten. */
    // __mmio (bPORT) = __mmio (bPORT) & ~msk;

    /* Portpin auf Eingang schalten. Der Eingang wird hochohmig.
       Der Kondensator wird nun ber das Poti geladen. Der Spannungsabfall
       ber dem Kondensator steigt und derjenige ber dem Poti sinkt.
       Der Portpin bleibt HIGH bis die Schwellwertspannung des Eingangs
       unterschritten wird, dies sind bei einer Versorgungsspannung von 5V
       ca 2.2V. */
    __mmio (bDDR) = __mmio (bDDR) & ~msk;


    /* Zaehlschleife.
       Wir zaehlen y so lange hoch bis der Kondensator unter die
       Schwellwertspannung geladen ist. */
    while (__mmio (bPIN) & msk && y < 65535) y++;

    // Zhlerwert zurckgeben
    return y;

} /* end of Poti */
