/****************************************************************************
    EntPrell.c
    ----------
    bungsprojekt zum Entprellen von mechanischen Kontakten.
    An Pin 0 von Port D wird ein mechanischer Taster zu Masse angeschlossen.
    An Port B schliessen wir ber Vorwiderstnde 4-8 LED's an, um die
    Anzahl Tastendrcke anzuzeigen. Achte darauf, dass nicht zu viel Strom
    aus dem AVR gezogen wird, also Widerstnde gengend gross whlen.
    Es wird nun mit einem Schleifenzhler geprft, wie lange der logische
    Zustand 0 am Pin anliegt. Erreicht der Schhleifenzhler einen bestimmten
    Wert (durch Versuche zu ermitteln) so wird der Tastendruck gewertet.

    Autor:    Christian Schifferle
    Datum:    Oktober 2002
    Software: AVR-GCC
    needed
****************************************************************************/

#include <io.h>	// Wir brauchen Zugriff auf die I/O's des Controllers

// Typendefinition fr ein BYTE
typedef unsigned char  BYTE;
typedef unsigned int   WORD;


#define	DELAY_LOOPS    250     // So viele Schleifendurchgaenge lang muss
                               // 'Taste gedrckt' anstehen.

// In der folgenden Variable wird der Zustand von Pin 0 an Port D
// zwischengespeichert um die Flanke zu erkennen.
// Zustzlich speichern wir in Bit 1 den aktuellen Status des Tasters.
// Wenn der Taster gedrckt ist (nach Entprellung) schreiben wir eine
// 1 in das Bit.
// Die Variable wird im SRAM abgelegt.
// Und zwar machen wir jetzt was ganz tolles und realiseren dies ber eine
// Bitvariable.
struct {
    BYTE bPin:1;     // Bit 0 fr das Signal am Pin 0.
    BYTE bStatus:1;  // Bit 1 fr das Speichern des Status der Taste
                     // Die verbleibenden 6 Bits der Variable werden nicht
                     // genutzt.
} bPortD;

// In der folgenden Variable werden die Anzahl Tastendrucke gespeichert.
// Die Variable wird im SRAM abgelegt.
BYTE	nKeyPress;

// Die folgende Variable dient als Schleifenzhler.
// Immer, wenn der Zustand 'Taste losgelassen' erkannt wird stellen wir
// den Zhler auf 0 zurck.
// Bei 'Taste gedrckt' inkrementieren wir den Zhler. Erreicht er einen
// durch Versuch ermittelten Zhlerstand so werten wir die Taste als
// gedrckt.
BYTE	nCounter;


int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{

    /* Wir brauchen Port B als Ausgang. Dazu setzen wir das Datenrichtungs-
       register von Port B.
       Da wir uns den externen Pull-Up Widerstand sparen wollen verwenden
       wir den Internen von Port D.
    */
    outp (0xff,DDRB);	// Port B als Ausgang
    outp (0x00,DDRD);	// Port D als Eingang
    outp (0x01,PORTD);	// Pull-Up auf Port D, Pin 0 aktivieren

    /* Hier beginnt die Endlosschleife. Dies knnte auch als
       while (1)
       definiert werden. Der Ausdruck for (;;) ist jedoch in C
       eine bliche Schreibweise fr sowas.
    */
    bPortD.bPin = 1;	// Variable initialisieren
    bPortD.bStatus = 0;
    nCounter = 0;

    for (;;) {
        /* Pin 0 von Port D abfragen, eine logische 0 heisst 'Taster gedrckt'.
            ACHTUNG: Es muss Register PIND abgefragt werden, nicht PORTD */
        if (bit_is_clear (PIND, PIND0)) {
            /* Der Taster ist gedrckt.
               Wir prfen jetzt, wie der Zustand zuvor war. */
            if (bPortD.bPin) {
                /* Schalter war vorher nicht gedrckt. */
                nCounter = 0;          // Zhler initialisieren
                bPortD.bPin = 0;       // Aktuellen Portzustand speichern
                bPortD.bStatus = 0;    // Status als 'Nicht gedrckt' initialisieren
            }
            else if (!bPortD.bStatus) {  // Nur wenn Status 'Nicht gedrckt'
                /* Zhler inkrementieren */
                if (++nCounter > DELAY_LOOPS) {
                    nKeyPress++;
                    /* und neuen Zhlerstand anzeigen */
                    outp (nKeyPress, PORTB);
                    // Tastenzustand merken
                    bPortD.bStatus = 1;	// Status merken
                }
            }
        }
        else {
            /* Taster ist nicht gedrckt */
            bPortD.bPin = 1;        // Aktuellen Zustand speichern
        }
   }
}
