/****************************************************************************
	Uhr.c
	-----

	Interruptgesteuerte Uhr

	Controller: AT90S2313

	Pinbelegung:	PB0...PB5	Minutenanzeige
					PB6			unbenutzt
					PB7			Taster zum Uhr einstellen
					PD0			unbenutzt
					PD1			unbenutzt
					PD2...PD6	Stundenanzeige

    Autor:    Christian Schifferle
    Datum:    Februar 2003
    Compiler: WinAVR

	nderungen:
	Version 1.0		Feb 2003
	  -	Erstausgabe.
****************************************************************************/

#include <avr\io.h>		// Wir brauchen Zugriff auf die I/O's des Controllers
#include <avr\signal.h>
#include <avr\interrupt.h>
#include <avr\eeprom.h>	// Und auf das Flash


#define F_CPU			 3686400		// CPU-Takt


#define FLAG_BUTTONCHECK	0x01	// Bitmaske fr Flagvariable

// Datentypdefinitionen
typedef unsigned char	BYTE;
typedef unsigned short	WORD;

volatile WORD nTicks=0;	// 3375 Ticks pro Minute bei 3686400 Hz
volatile BYTE nMin=0;	// Minutenzhler
volatile BYTE nHour=0;	// Stundenzhler

// In der Variable wFlags knnen bis zu 8 globale Flags gespeichert
// werden. Wir bentigen nur 1 Flag.
volatile BYTE wFlags=0;


void MinutenZaehler (void)
/***********************************************************************
	Diese Funktion wird bei jedem Minutenwechsel aufgerufen oder, wenn
	die Einstelltaste gedrckt ist.
***********************************************************************/
{
	nMin++;
	if (nMin == 60) {
		nMin = 0;
		nHour++;
		if (nHour == 24) nHour = 0;
	}
	// Anzeige der Minuten auf Port B. Da der Wert niemals grsser als
	// 59 werden kann bleiben die beiden hchstwertigen Bits immer auf 0.
	// Der Pull-Up Widerstand auf Pin 7 muss aber eingeschaltet bleiben,
	// also setzen wir das entsprechende Bit gleich auch mit.
	outp (nMin | 0x80, PORTB);
	// Anzeige der Stunden. Durch das Linksschieben bleiben die zwei
	// niederwertigsten Bits immer auf 0.
	outp (nHour << 2, PORTD);

} /* end of MinutenZaehler */



// Wir werten hier den Timer overflow Interrupt vom Timer 1 aus
SIGNAL(SIG_OVERFLOW1)
{
	nTicks++;
	if (nTicks == 3375) {
		MinutenZaehler ();
		nTicks = 0;
	}

	// Hier wird das Flag fr die Tastaturabfrage gesetzt, und zwar
	// bei jedem 25. Durchlauf
	if (!(nTicks % 25)) wFlags |= FLAG_BUTTONCHECK;

} /* end of SIGNAL(SIG_OVERFLOW1) */



int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{

	// Pull-Up Widerstand Port B, Pin 7 aktivieren
	// 1000 0000 = 0x80
	outp (0x80, PORTB);

    // Port B, Pin 0...5 als Ausgang konfigurieren
	// 0011 1111 = 0x3f
    outp (0x3f, DDRB);

    // Timer/Counter1 (16-Bit-Timer) initialisieren
	outp (0, TCCR1A);
    outp (1<<CS10, TCCR1B); // Vorzaehler = 1, pro Minute 3375 Durchlufe

	// Timer-Interrupt aktivieren
	timer_enable_int (_BV(TOIE1));

	// Interrupts zulassen
	sei();

    for (;;) {
		// Hier knnen wir allerlei unwichtiges machen

		if (wFlags & FLAG_BUTTONCHECK) {	// Entprellung ber globales Flag
			// Tasten berprfen. Da dies nur ca. 3 mal pro Sekunde passiert
			// haben wir automatisch auch eine Entprellung und eine Dauerstellung
			if (bit_is_clear (PINB, PINB7)) {
				MinutenZaehler ();
			}
			// Flag lschen, damit wirklich nur einmal inkrementiert wird
			wFlags &= ~FLAG_BUTTONCHECK;
		}
    }

} /* end of main */
